# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from six import with_metaclass
from azure.core import CaseInsensitiveEnumMeta


class DimensionScope(with_metaclass(CaseInsensitiveEnumMeta, str, Enum)):
    """The scope at which the quota is applied.
    """

    WORKSPACE = "Workspace"
    SUBSCRIPTION = "Subscription"

class JobStatus(with_metaclass(CaseInsensitiveEnumMeta, str, Enum)):
    """The job status.
    """

    WAITING = "Waiting"
    EXECUTING = "Executing"
    SUCCEEDED = "Succeeded"
    FAILED = "Failed"
    CANCELLED = "Cancelled"

class JsonPatchOperation(with_metaclass(CaseInsensitiveEnumMeta, str, Enum)):
    """The operation to be performed.
    """

    ADD = "add"
    REMOVE = "remove"
    REPLACE = "replace"
    MOVE = "move"
    COPY = "copy"
    TEST = "test"

class MeterPeriod(with_metaclass(CaseInsensitiveEnumMeta, str, Enum)):
    """The time period in which the quota's underlying meter is accumulated. Based on calendar year.
    'None' is used for concurrent quotas.
    """

    NONE = "None"
    MONTHLY = "Monthly"

class ProviderAvailability(with_metaclass(CaseInsensitiveEnumMeta, str, Enum)):
    """Provider availability.
    """

    AVAILABLE = "Available"
    DEGRADED = "Degraded"
    UNAVAILABLE = "Unavailable"

class TargetAvailability(with_metaclass(CaseInsensitiveEnumMeta, str, Enum)):
    """Target availability.
    """

    AVAILABLE = "Available"
    DEGRADED = "Degraded"
    UNAVAILABLE = "Unavailable"
